# $Id: metaclient.py,v 1.11 2002/02/17 21:57:08 marijn Exp $

# An air traffic simulation game. 
# Copyright (C) 2000,1,2  Marijn Vriens <marijn@sanity.dhs.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import setup
import os
import urllib

os.environ["IDLPATH"] = setup.dir_idl
import CORBA, Meta, Meta__POA

import absclient
 
def MetaClient(fifo=None): # singleton constructor
    if _MetaClient.instance == None:
        _MetaClient.instance = _MetaClient(fifo)
    return _MetaClient.instance

class _MetaClient(absclient.AbsClient):
    instance = None
    def __init__(self, fifo):
        absclient.AbsClient.__init__(self, fifo)
    def _genExceptionNameClassLink(self):
        absclient.AbsClient._genExceptionNameClassLink(self)
        self._exceptionHandlerClassDict[Meta.NoExist] = "NoExist"
    # *_impl called by the apply of run()
    def connect_impl(self, url):
        ior = urllib.urlopen(url).readline()
        self.handle = self._orb.string_to_object(ior)
    def getServerList_impl(self):
        v = setup.version()
        dat = self.handle.getServerList(v)
        return (dat,)
    def getServerData_impl(self, ior):
        d = self.handle.getServerData(ior)
        return (d, ior)
    def latestVersion_impl(self):
        d = self.handle.latestVersion()
        return d

# $Log: metaclient.py,v $
# Revision 1.11  2002/02/17 21:57:08  marijn
# Added generalized exception handling w/ callbacks
#
# Revision 1.10  2002/02/02 17:23:57  marijn
# Updated copyright information
#
# Revision 1.9  2002/01/24 14:13:19  marijn
# small fix
#
# Revision 1.8  2002/01/16 18:09:54  marijn
# Changed from named-pipes to normal ones.
# Added AbsClient.executeWithDelay() that executes the event later.
# Fixed some inter-threading bugs. the Exception de-syncing is still here.
#
# Revision 1.7  2002/01/14 15:42:45  marijn
# Added missing module ref
#
# Revision 1.6  2001/12/25 02:02:14  marijn
# Various fixes.
#
# Revision 1.5  2001/12/11 03:53:23  marijn
# Split code off, moved common client code to absclient.py
#
# Revision 1.4  2001/12/10 22:15:14  marijn
# Changed threading implementation
#
# Revision 1.3  2001/12/10 00:15:25  marijn
# Added _MethodProxy() and various bug fixes.
#
# Revision 1.2  2001/12/06 06:49:43  marijn
# MetaClient now has a seperate thread.
#
# Revision 1.1  2001/12/01 19:39:48  marijn
# Renamed clientmeta.py to metaclient.py
#
# Revision 1.2  2001/11/19 00:25:39  marijn
# py merged back into HEAD
#
# Revision 1.1.2.4  2001/11/16 04:00:12  marijn
# Client connects to simserver
#
# Revision 1.1.2.3  2001/11/11 08:03:56  marijn
# Shows registered simServers in selector window.
#
# Revision 1.1.2.2  2001/11/07 12:02:16  marijn
# More Ui code, moved meta server invocation to functional location
#
# Revision 1.1.2.1  2001/11/05 05:34:58  marijn
# Client now talks to meta server.
#
