# $Id: radardisplay.py,v 1.8 2002/02/18 13:01:37 marijn Exp $

# An air traffic simulation game. 
# Copyright (C) 2000,1,2  Marijn Vriens <marijn@sanity.dhs.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import setup

import math
import gnome.ui 
import gnome.affine
import gtk 
import GTK
import GDK

import abswindow

class RadarDisplay(abswindow.AbsWindow):
    WindowNickName = "display"
    def _build(self):
        self._root = self._window.root()
        self._canvasGroups = self._genCanvasGroups(self._root)
        self._setupCanvas(self._window.size_request())
        self._controlledVisual = None
        self.showIntroScreen()
        #self.showRadarGrid()
    def hide(self):
        pass
    def _setupCanvas(self, DisplayDimesions):
        self._width = DisplayDimesions[0]
        self._height= DisplayDimesions[1]
        self._window.set_scroll_region(0, 0, self._width, self._height)
        self._window.set_pixels_per_unit(1)
    def getCanvasSize(self):
        return self._width, self._height
    def getRadarSize(self):
        return -180, -90, 180, 90
    def getCanvasGroup(self, name):
        return self._canvasGroups[name]
    def showRadarGrid(self):
        (width, height) = self.getCanvasSize()
        group = self.getCanvasGroup('grid')
        radar = self.getRadarSize()
        max = abs(radar[0] - radar[2])
        for i in range(0, max, 20): # longitude lines. 
            group.add("line", points=((width/float(max))*i, 0,
                                      (width/float(max))*i, height),
                      fill_color="green", width_pixels=1)
        max = abs(radar[1] - radar[3])
        for i in range(0, max, 10): # latitude lines. 
            group.add("line", points=(0, (height/float(max))*i,
                                      width, (height/float(max))*i),
                      fill_color="green", width_pixels=1)
    def showIntroScreen(self):
        group = self.getCanvasGroup("intro")
        top   =  0
        left  =  0
        (bottom, right)= self.getCanvasSize()
        group.add("ellipse", x1=left, y1=top, x2=right, y2=bottom,
                       outline_color="green", fill_color='black',
                       width_units=1.0)
        group.add("text", text=setup.package, x=(left+right)/2, y=(top+bottom)/2-10,
                       font='-*-fixed-*-*-*-*-20-*-*-*-*-*-*-*',
                       anchor=GTK.ANCHOR_CENTER,
                       justification=GTK.JUSTIFY_CENTER, fill_color='white')
        group.add("text", text=setup.showVersion(), x=(left+right)/2, y=(top+bottom)/2+10,
                       font='-*-fixed-*-*-*-*-20-*-*-*-*-*-*-*',
                       anchor=GTK.ANCHOR_CENTER,
                       justification=GTK.JUSTIFY_CENTER, fill_color='white')
    def removeIntroScreen(self):
        self.getCanvasGroup("intro").hide()
    def visualBeingControlled(self, visual):
        if self._controlledVisual:
            self._controlledVisual.noLongerControlled()
        self._controlledVisual = visual
    def _genCanvasGroups(self, root):
        groups = {}
        groups['intro'] = self._root.add("group") # lowermost group
        groups['grid']  = self._root.add("group") 
        groups['plane'] = self._root.add("group")
        return groups
   
class PlaneVisual:
    # Should show:                         Example
    # Flight number                        AA1234
    # Speed (knots)                        230
    # Flight Level (x 10000ft)             33
    # Heading                              240
    def __init__(self, data):
        assert(data != None)
        self._data = data
        self._activeController = None
        self._display = abswindow.WindowList().getWindow('display')
        self._parentGroup = self._display.getCanvasGroup('plane')
        self._visual = self._generateVisual(self._parentGroup)
        (posx, posy) = self._calcVisualLocation()
        self._posx = posx
        self._posy = posy
        self._visual.move(posx, posy)
        self._visual.connect("event", self.handleCanvasMouseClick, self)
    def update(self):
        (posx, posy) = self._calcVisualLocation()
        self._moveVisual(self._visual, posx, posy)
        self._updateText()
        self._updateLine()
        if(self._activeController):
            self._activeController.updateDisplay()
    def noLongerControlled(self):
        self._activeController = None
        self._unMarkAsControlled()
    def beingControlled(self):
        wl = abswindow.WindowList()
        self._activeController = wl['control']
        self._activeController.setEntityToDisplay(self._data)
        self._display.visualBeingControlled(self)
        self._markAsControlled()
    def _generateVisual(self, parent):
        lineSpacing = 8
        visual = parent.add("group")
        self._controlMarker = self._genControlMarker(visual, 10)
        self._genLine(visual)
        textGroup = visual.add("group")
        self._visNumber = self._genText(textGroup, self._data.flightNumber(),
                                        0, 0)
        self._visSpeed  = self._genText(textGroup,
                                        self._formatSpeedText(self._data.speed()),
                                        0, lineSpacing)
        self._visHeight = self._genText(textGroup,
                                        self._formatHeightText(self._data.height()),
                                        0, lineSpacing*2)
        self._visHeading= self._genText(textGroup,
                                        self._formatHeadingText(self._data.heading()),
                                        0, lineSpacing*3)
        return visual
    def _markAsControlled(self):
        print self, "marked as controlled"
        self._controlMarker.show()
    def _unMarkAsControlled(self):
        print self, "REMOVED as controlled"
        self._controlMarker.hide()
    def _genControlMarker(self, group, size):
        marker = group.add("ellipse", x1=-size, y1=-size, x2=size, y2=size,
                           fill_color="gray")
        marker.hide()
        return marker
    def _genLine(self, group):
        lengthOfSpeed = self._data.speed() * 1
        headingPi = _degrees2pi(self._data.heading())
        y = math.cos(headingPi) * lengthOfSpeed * -1
        x = math.sin(headingPi) * lengthOfSpeed
        self._visLine = group.add("line", points=(0,0, x,y),
                                  fill_color="red", width_units=2) 
    def _genText(self, group, text, x, y):
        if type(text) != type(' '):
            text = str(int(text))
        r = group.add("text", text=text, x=x, y=y,
                  font='fixed', anchor=GTK.ANCHOR_EAST, fill_color='black')
        return r
    def _updateLine(self):
        lengthOfSpeed = self._data.speed() * 10
        headingPi = _degrees2pi(self._data.heading())
        y = math.cos(headingPi) * lengthOfSpeed * -1
        x = math.sin(headingPi) * lengthOfSpeed
        self._visLine.set(points=(0,0, x,y))
    def _updateText(self):
        self._visSpeed.set(text=self._formatSpeedText(self._data.speed()))
        self._visHeight.set(text=self._formatHeightText(self._data.height()))
        self._visHeading.set(text=self._formatHeadingText(self._data.heading()))
    def _calcVisualLocation(self):
        (width, height) = self._display.getCanvasSize()
        if(self._data.latitude() < -90) or (self._data.latitude() > 90):
            raise "DataOutOfBounds"
        if(self._data.longitude() < -180) or (self._data.latitude() > 180):
            raise "DataOutOfBounds"
        radar = self._display.getRadarSize()
        longiRange = abs(radar[0]-radar[2])
        longiOffset= radar[0]
        latiRange  = abs(radar[1]-radar[3])
        latiOffset = radar[1]
        xratio = (width / float(longiRange))
        yratio = (height/ float(latiRange) )
        xloc = xratio * ( self._data.longitude() - longiOffset)
        yloc = yratio * ( (-1*self._data.latitude())  - latiOffset)
        return xloc, yloc
    def _moveVisual(self, visual, x, y):
        dx = x - self._posx
        self._posx = x
        dy = y - self._posy
        self._posy = y
        visual.move(dx, dy)
    def _formatHeadingText(self, nr):
        return str(int(round(nr)))
    def _formatSpeedText(self, nr):
        return str(int(round(nr)))
    def _formatHeightText(self, nr):
        return str(int(round(nr/10000.0)))

    # Callbacks
    def handleCanvasMouseClick(self, canvasItem, event, visualItem):
        if event.type == GDK.BUTTON_PRESS:
            visualItem.beingControlled()

def _degrees2pi(deg):
    return deg * ( math.pi / 180 )
def showStructure(top, t=None):
    if(t == None):
        t = 0
    print "\t" * t + str(top)
    for i in top.children():
        if(i.__class__ == top.__class__):
            showStructure(i, t+1)
        else:
            d = t + 1
            print "\t" * d + str(i)

# $Log: radardisplay.py,v $
# Revision 1.8  2002/02/18 13:01:37  marijn
# Silenced some debug info
#
# Revision 1.7  2002/02/10 18:09:53  marijn
# Improved order information feedback
#
# Revision 1.6  2002/02/07 16:25:21  marijn
# small fixes
#
# Revision 1.5  2002/02/07 12:54:06  marijn
# Reordered windows, Added plane ordering functionality
#
# Revision 1.4  2002/02/02 17:23:57  marijn
# Updated copyright information
#
# Revision 1.3  2002/02/01 23:54:42  marijn
# Added graphics to client
#
# Revision 1.2  2002/01/24 14:14:29  marijn
# redesigned object model
#
# Revision 1.1  2002/01/14 15:38:58  marijn
# Initial support for data display
#
