# $Id: sim.py,v 1.6 2002/02/07 12:54:06 marijn Exp $

# An air traffic simulation game. 
# Copyright (C) 2000,1,2  Marijn Vriens <marijn@sanity.dhs.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import UserDict

import setup
import ui
import simclient
import abswindow
import radardisplay
import CORBA, SimulationServer

class Simulation:
    def __init__(self, ior, nick):
        self._nick = nick
        self._display = abswindow.WindowList().getWindow('display')
        self._handle  = simclient.SimClient()
        self._info    = InfoDict()
        self._showNeededWindows()
        self._handle.connect(self.connected_cb, ior, nick)
    def connected_cb(self, id):
        print "Simulation::connected_cb called"
        self._display.removeIntroScreen()
        self._display.showRadarGrid()
        self._connId = id
        if(self._connId == 0):
            raise "NickTaken"
        self._handle.getUpdate(self.getUpdate_cb)
    def getUpdate_cb(self, update):
        map(self._info.objEntered,   update.objsEntryList)
        map(self._subscribeToObject, update.objsEntryList)
        map(self._info.objLeft, update.objsLeaveList)
        map(self._info.updateSubscribedObject, update.planeUpdateList)
        self._handle.executeWithDelay(1, 'getUpdate', self.getUpdate_cb)
    def _subscribeToObject(self, obj, *arg):
        #print "Simulation::_subscribeToObject: id = %d" % (obj.id)
        self._handle.subscribe(None, obj.id)
    def _showNeededWindows(self):
        wl = abswindow.WindowList().showWindow('control')

class InfoDict(UserDict.UserDict):
    def __init__(self):
        UserDict.UserDict.__init__(self)
    def updateSubscribedObject(self, updateData):
        if not self.data.has_key(updateData.id):
            raise "CannotUpdateUnsubscribedObject"
        obj = self.data[updateData.id]
        if not callable(obj.update):
            raise "ShouldNotHappen"
        obj.update(updateData)
    def objEntered(self, obj):
        #print "Plane entered id = %d" % obj.id
        if self.data.has_key(obj.id):
            raise "ObjectIdAlreadyUsed"
        i = ItemFactory(obj)
        self.data[i.id()] = i
    def objLeft(self, obj):
        #print "Plane left id = %d" % obj.id
        if not self.data.has_key(obj.id):
            raise "ObjectNotKnown"
        del self.data[obj.id]
    def __setitem__(self):
        raise "NOT_SUPPORTED"

def ItemFactory(item):
    assert(item.__class__ == SimulationServer.SimObjType)
    if item.objTypeName == 'plane':
        return PlaneData(item)
    if item.objTypeName == 'generic':
        return _Item(item)
    else:
        raise "entityObjTypeUnknown", "\"%s\" is not known" % item.objTypeName

class _Item:
    def __init__(self, entityData):
        self._id = entityData.id
        self._latitude = entityData.pos.lat
        self._longitude = entityData.pos.lon
        self._height = entityData.pos.vert
    def id(self):
        return self._id
    def update(self, updateData):
        self._latitude = updateData.pos.lat
        self._longitude= updateData.pos.lon
        self._height   = updateData.pos.vert
    def latitude(self):
        return self._latitude
    def longitude(self):
        return self._longitude
    def height(self):
        return self._height    

class PlaneData(_Item):
    def __init__(self, entityData):
        _Item.__init__(self, entityData)
        try:
            self._heading  = entityData.pos.heading
        except(AttributeError):
            self._heading = 0
        try:
            self._speed = entityData.pos.speed
        except(AttributeError):
            self._speed = 0
        td = TypeDict()
        self._type = td['generic']
        self._visual = radardisplay.PlaneVisual(self)
        #print "%d: %d,%d,%d - %d x %d" % (self.id(), self.latitude(),
        #                                   self.longitude(), self.height(),
        #                                   self.heading(), self.speed())
    def flightNumber(self):
        return "AA1234"
    def heading(self):
        return self._heading
    def speed(self):
        return self._speed
    def update(self, entityData):
        assert(entityData.id == self.id())
        self._latitude = entityData.pos.lat
        self._longitude= entityData.pos.lon
        self._height   = entityData.pos.vert
        self._heading  = entityData.pos.heading
        self._speed    = entityData.pos.speed
        #print "PlaneData::update: %s @ %s, %s, %s\n --- h%s , s%s" % (
        #    repr(id(self)), self.latitude(),
        #    self.longitude(), self.height(),
        #    self.heading(), self.speed())
        self._visual.update()
    def getType(self):
        return self._type

def TypeDict():
    if _TypeDict.instance == None:
        _TypeDict.instance = _TypeDict()
    return _TypeDict.instance

class _TypeDict(UserDict.UserDict):
    instance = None
    def __init__(self):
        UserDict.UserDict.__init__(self)
        self.data['generic'] = PlaneTypeInfo()
    def load(self):
        pass
    def __setitem__(self):
        raise "NOT_SUPPORTED"

class PlaneTypeInfo:
    def maxSpeed(self):
        return 100
    def minSpeed(self):
        return 1
    def maxAltitude(self):
        return 600
    def minAltitude(self):
        return 0
        
# $Log: sim.py,v $
# Revision 1.6  2002/02/07 12:54:06  marijn
# Reordered windows, Added plane ordering functionality
#
# Revision 1.5  2002/02/02 17:23:57  marijn
# Updated copyright information
#
# Revision 1.4  2002/02/01 23:54:42  marijn
# Added graphics to client
#
# Revision 1.3  2002/01/24 14:14:29  marijn
# redesigned object model
#
# Revision 1.2  2002/01/16 18:09:54  marijn
# Changed from named-pipes to normal ones.
# Added AbsClient.executeWithDelay() that executes the event later.
# Fixed some inter-threading bugs. the Exception de-syncing is still here.
#
# Revision 1.1  2002/01/14 15:38:58  marijn
# Initial support for data display
#
