# $Id: testradardisplay.py,v 1.3 2002/02/18 10:49:08 marijn Exp $

# An air traffic simulation game. 
# Copyright (C) 2000,1,2  Marijn Vriens <marijn@sanity.dhs.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import unittest

import fakegtk
import fakedata

import sim
import radardisplay

def suite():
    testplanevisual = unittest.TestSuite()
    testplanevisual.addTest(testPlaneVisual('init'))
    testplanevisual.addTest(testPlaneVisual('_calcVisualLocation'))


    testradardisplay = unittest.TestSuite()
    testradardisplay.addTest(testRadarDisplay('init'))
    testradardisplay.addTest(testRadarDisplay('_setupCanvas'))
    testradardisplay.addTest(testRadarDisplay('getCanvasSize'))
    testradardisplay.addTest(testRadarDisplay('getRadarSize'))
    testradardisplay.addTest(testRadarDisplay('getCanvasGroup'))
    testradardisplay.addTest(testRadarDisplay('showRadarGrid'))
    testradardisplay.addTest(testRadarDisplay('showIntroScreen'))
    testradardisplay.addTest(testRadarDisplay('removeIntroScreen'))


    testglobal = unittest.TestSuite()
    testglobal.addTest(testGlobal('degrees2pi'))

    suite = unittest.TestSuite()
    suite.addTest(testglobal)
    suite.addTest(testplanevisual)
    suite.addTest(testradardisplay)
    return suite

class testRadarDisplay(unittest.TestCase):
    def setUp(self):
        fakedata.setUpUi()
        self.obj = radardisplay.RadarDisplay(fakegtk.fakeCanvas())
    def init(self):
        assert(self.obj._window)
        assert(self.obj._root)
    def _setupCanvas(self):
        self.obj._setupCanvas( (700, 700), )
    def getCanvasSize(self):
        self.obj._setupCanvas( (100, 150), )
        (width, height) = self.obj.getCanvasSize()
        assert(width == 100)
        assert(height== 150)
    def getRadarSize(self):
        self.obj.getRadarSize()
    def getCanvasGroup(self):
        assert(self.obj.getCanvasGroup('grid'))
        assert(self.obj.getCanvasGroup('plane'))
        try:
            self.obj.getCanvasGroup('foo')
        except(KeyError):
            pass
        else:
            self.fail()
    def showRadarGrid(self):
        self.obj.showRadarGrid()
    def showIntroScreen(self):
        self.obj.showIntroScreen()
    def removeIntroScreen(self):
        self.obj.removeIntroScreen()        

class testPlaneVisual(unittest.TestCase):
    def setUp(self):
        fakedata.setUpUi()
        radardisplay.RadarDisplay(fakegtk.fakeCanvas())
        corbaData = fakedata.genPlaneType(42, 10, 10, 10,
                                          100, 90)
        self.data = sim.PlaneData(corbaData)
        self.obj = radardisplay.PlaneVisual(self.data)
    def init(self):
        assert(self.obj._data)
        assert(self.obj._display)
        assert(self.obj._posx)
        assert(self.obj._posy)
    def _calcVisualLocation(self):
        corbaData = fakedata.genPlaneTypeSet(42, 90, -180, 10, 100, 90)
        (xcz, ycz) = self.obj._display.getCanvasSize()
        self.data.update(corbaData)
        r = self.obj._calcVisualLocation()
        assert(r[0] == 0)
        assert(r[1] == 0)
        corbaData = fakedata.genPlaneTypeSet(42, 0, 0, 10, 100, 90)
        self.data.update(corbaData)
        r = self.obj._calcVisualLocation()
        assert(r[0] == xcz/2.0)
        assert(r[1] == ycz/2.0)
        corbaData = fakedata.genPlaneTypeSet(42, -90, 180, 10, 100, 90)
        self.data.update(corbaData)
        r = self.obj._calcVisualLocation()
        assert(r[0] == xcz)
        assert(r[1] == ycz)
        
        
class testGlobal(unittest.TestCase):
    def degrees2pi(self):
        assert(radardisplay._degrees2pi(0)   == 0)
        assert(radardisplay._degrees2pi(180) == radardisplay.math.pi)
        assert(radardisplay._degrees2pi(360) == radardisplay.math.pi*2)

# $Log: testradardisplay.py,v $
# Revision 1.3  2002/02/18 10:49:08  marijn
# Updated copyright information
#
# Revision 1.2  2002/02/07 12:51:36  marijn
# Redid window referencing
#
# Revision 1.1  2002/02/01 23:55:31  marijn
# initial tests added
#
