# $Id: testsim.py,v 1.3 2002/02/18 10:49:08 marijn Exp $

# An air traffic simulation game. 
# Copyright (C) 2000,1,2  Marijn Vriens <marijn@sanity.dhs.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import unittest

import fakedata 
import fakegtk

import sim
import radardisplay

def suite():
    testsimulation = unittest.TestSuite()
    testsimulation.addTest(testSimulation('init'))

    testinfodict = unittest.TestSuite()
    testinfodict.addTest(TestInfoDict('objEntered'))
    testinfodict.addTest(TestInfoDict('objLeft'))
    testinfodict.addTest(TestInfoDict('updateSubscribedObject'))

    test_item = unittest.TestSuite()
    test_item.addTest(test_Item('init'))
    test_item.addTest(test_Item('id'))

    testplanedata = unittest.TestSuite()
    testplanedata.addTest(testPlaneData('init'))
    testplanedata.addTest(testPlaneData('update'))

    suite = unittest.TestSuite()
    suite.addTest(test_item)
    suite.addTest(testplanedata)
    suite.addTest(testinfodict)
    suite.addTest(testsimulation)
    return suite

class testSimulation(unittest.TestCase):
    def setUp(self):
        fakedata.setUpUi()
        fakedata.fakeWindow('display')        
        sim.simclient.SimClient = fakedata.FakeSimClient
        self.obj = sim.Simulation('strange string', 'tester')
        data1 = sim.SimulationServer.UpdateType()
        data1.planeUpdateList = []
        data1.objsEntryList = []
        data1.objsEntryList.append(fakedata.genSimObjType(id=42))
        data1.objsEntryList.append(fakedata.genSimObjType(id=203))
        data1.objsEntryList.append(fakedata.genSimObjType(id=23))
        data1.objsEntryList.append(fakedata.genSimObjType(id=38))
        data1.objsLeaveList = []
        self.obj.getUpdate_cb(data1)
        data2 = sim.SimulationServer.UpdateType()
        data2.planeUpdateList = []
        data2.objsEntryList = []
        data2.planeUpdateList.append(fakedata.genPlaneType(id=42))
        data2.planeUpdateList.append(fakedata.genPlaneType(id=203))
        data2.objsLeaveList = []
        data2.objsLeaveList.append(fakedata.genPlaneType(id=23))
        self.obj.getUpdate_cb(data2)
    def init(self):
        assert(len(self.obj._info) == 3)
        assert(self.obj._info[42])
        assert(self.obj._info[38].id() == 38)
        assert(self.obj._info[203].id() == 203)

class TestInfoDict(unittest.TestCase):
    def setUp(self):
        fakedata.setUpUi()
        sim.radardisplay.RadarDisplay(fakegtk.fakeCanvas())
        self.obj = sim.InfoDict()
        self.list = []
        self.list.append(fakedata.genSimObjType(id=42 ))
        self.list.append(fakedata.genSimObjType(id=200))
        self.list.append(fakedata.genSimObjType(id=34))
    def objEntered(self):
        self.obj.objEntered(self.list[0])
        self.obj.objEntered(self.list[1])
        self.obj.objEntered(self.list[2])
        assert(self.obj[42])
        assert(self.obj[34])
        assert(len(self.obj) == 3)
        assert(self.obj[200].id() == self.list[1].id)
        f = fakedata.genSimObjType(id=200)
        try:
            self.obj.objEntered(f)
        except "ObjectIdAlreadyUsed":
            pass
        else:
            self.fail()
    def objLeft(self):
        self.obj.objEntered(self.list[0])
        self.obj.objEntered(self.list[1])
        assert(self.obj[42])
        d = fakedata.genSimObjType(id=42)
        self.obj.objLeft(d)
        assert(not self.obj.has_key(42))
        assert(self.obj[200])
        d = fakedata.genSimObjType(id=42)
        try:
            self.obj.objLeft(d)
        except "ObjectNotKnown":
            pass
        else:
            self.fail()
    def updateSubscribedObject(self):
        self.obj.objEntered(self.list[0])
        self.obj.objEntered(self.list[1])
        self.obj.objEntered(self.list[2])
        l = []
        l.append(fakedata.genPlaneType(id=42))
        l.append(fakedata.genPlaneType(id=200))
        assert(self.obj[42].latitude() == self.list[0].pos.lat)
        self.obj.updateSubscribedObject(l[0])
        self.obj.updateSubscribedObject(l[1])
        assert(self.obj[42].latitude() == l[0].pos.lat)
        assert(len(self.obj) == 3)

class test_Item(unittest.TestCase):
    def setUp(self):
        data = fakedata.genSimObjType(id=42, lat=40.1, lon=-0.23, vert=93.3)
        assert(data.objTypeName == 'generic')
        self.obj = sim.ItemFactory(data)
    def init(self):
        assert(self.obj.latitude()  == 40.1)
        assert(self.obj.longitude() == -0.23)
        assert(self.obj.height()    == 93.3)
    def id(self):
        assert(self.obj.id() == 42)
        
class testPlaneData(unittest.TestCase):
    def setUp(self):
        fakedata.setUpUi()
        radardisplay.RadarDisplay(fakegtk.fakeCanvas())
        data = fakedata.genSimObjType(42, 40.1, -0.23, 93.3, type='plane')
        self.obj = sim.ItemFactory(data)
        data = fakedata.genPlaneType(42, -40.1, -0.23, 40.9, 239.503, 30.3)
        self.obj.update(data)
    def init(self):
        assert(type(self.obj.flightNumber()) == type(""))
        assert(len(self.obj.flightNumber()) == 6)
        assert(self.obj.id()       == 42)
        assert(self.obj.longitude()== -0.23)
        assert(self.obj.latitude() == -40.1)
        assert(self.obj.heading()  == 30.3)
        assert(self.obj.speed()    == 239.503)
        assert(self.obj.height()   == 40.9)
    def update(self):
        data = fakedata.genPlaneType(42, -90, 180, 10, 10, 90)
        self.obj.update(data)
            
# $Log: testsim.py,v $
# Revision 1.3  2002/02/18 10:49:08  marijn
# Updated copyright information
#
# Revision 1.2  2002/02/07 12:51:36  marijn
# Redid window referencing
#
# Revision 1.1  2002/02/01 23:55:31  marijn
# initial tests added
#
